class ControllerMLFlow:
    def __init__(self):
        pass
        # self.command_dir_logs = 'mkdir ~/mlflow/mllogs'
        # self.command_dir_experiments = 'mkdir ~/mlflow/mlruns'

    def create_dir():
        pass

    def run_mlflow(self):
        pass
